@@ -16,16 +16,18 @@ from ruamel.yaml.compat import ordereddict, PY2, string_types
 from ruamel.yaml.scalarstring import ScalarString
 
 if PY2:
-    from collections import MutableSet, Sized, Set
+    from collections import MutableSet, Sized, Set, MutableMapping
 else:
-    from collections.abc import MutableSet, Sized, Set
+    from collections.abc import MutableSet, Sized, Set, MutableMapping
 
 if False:  # MYPY
-    from typing import Any, Dict, Optional, List, Union  # NOQA
+    from typing import Any, Dict, Optional, List, Union, Optional  # NOQA
 
+# fmt: off
 __all__ = ["CommentedSeq", "CommentedKeySeq",
            "CommentedMap", "CommentedOrderedMap",
            "CommentedSet", 'comment_attrib', 'merge_attrib']
+# fmt: on
 
 comment_attrib = '_yaml_comment'
 format_attrib = '_yaml_format'
@@ -38,7 +40,7 @@ tag_attrib = '_yaml_tag'
 class Comment(object):
     # sys.getsize tested the Comment objects, __slots__ makes them bigger
     # and adding self.end did not matter
-    __slots__ = 'comment', '_items', '_end', '_start',
+    __slots__ = 'comment', '_items', '_end', '_start'
     attrib = comment_attrib
 
     def __init__(self):
@@ -56,9 +58,8 @@ class Comment(object):
         if bool(self._end):
             end = ',\n  end=' + str(self._end)
         else:
-            end = ''
-        return "Comment(comment={0},\n  items={1}{2})".format(
-            self.comment, self._items, end)
+            end = ""
+        return 'Comment(comment={0},\n  items={1}{2})'.format(self.comment, self._items, end)
 
     @property
     def items(self):
@@ -93,7 +94,7 @@ def NoComment():
 
 
 class Format(object):
-    __slots__ = '_flow_style',
+    __slots__ = ('_flow_style',)
     attrib = format_attrib
 
     def __init__(self):
@@ -126,7 +127,7 @@ class LineCol(object):
         # type: () -> None
         self.line = None
         self.col = None
-        self.data = None  # type: Union[None, Dict[Any, Any]]
+        self.data = None  # type: Optional[Dict[Any, Any]]
 
     def add_kv_line_col(self, key, data):
         # type: (Any, Any) -> None
@@ -163,7 +164,7 @@ class LineCol(object):
 
 
 class Anchor(object):
-    __slots__ = 'value', 'always_dump',
+    __slots__ = 'value', 'always_dump'
     attrib = anchor_attrib
 
     def __init__(self):
@@ -174,7 +175,8 @@ class Anchor(object):
 
 class Tag(object):
     """store tag information for roundtripping"""
-    __slots__ = 'value',
+
+    __slots__ = ('value',)
     attrib = tag_attrib
 
     def __init__(self):
@@ -231,6 +233,7 @@ class CommentedBase(object):
         """
         from .error import CommentMark
         from .tokens import CommentToken
+
         pre_comments = self._yaml_get_pre_comment()
         if comment[-1] == '\n':
             comment = comment[:-1]  # strip final newline if there
@@ -238,8 +241,9 @@ class CommentedBase(object):
         for com in comment.split('\n'):
             pre_comments.append(CommentToken('# ' + com + '\n', start_mark, None))
 
-    def yaml_set_comment_before_after_key(self, key, before=None, indent=0,
-                                          after=None, after_indent=None):
+    def yaml_set_comment_before_after_key(
+        self, key, before=None, indent=0, after=None, after_indent=None
+    ):
         # type: (Any, Any, Any, Any, Any) -> None
         """
         expects comment (before/after) to be without `#` and possible have multiple lines
@@ -250,7 +254,7 @@ class CommentedBase(object):
         def comment_token(s, mark):
             # type: (Any, Any) -> Any
             # handle empty lines as having no comment
-            return CommentToken(('# ' if s else '') + s + '\n', mark, None)
+            return CommentToken(('# ' if s else "") + s + '\n', mark, None)
 
         if after_indent is None:
             after_indent = indent + 2
@@ -261,7 +265,7 @@ class CommentedBase(object):
         start_mark = CommentMark(indent)
         c = self.ca.items.setdefault(key, [None, [], None, None])
         if before == '\n':
-            c[1].append(comment_token('', start_mark))
+            c[1].append(comment_token("", start_mark))
         elif before:
             for com in before.split('\n'):
                 c[1].append(comment_token(com, start_mark))
@@ -291,6 +295,7 @@ class CommentedBase(object):
         """
         from .tokens import CommentToken
         from .error import CommentMark
+
         if column is None:
             column = self._yaml_get_column(key)
         if comment[0] != '#':
@@ -354,6 +359,7 @@ class CommentedBase(object):
 
     def copy_attributes(self, t, deep=False):
         # type: (Any, bool) -> None
+        # fmt: off
         for a in [Comment.attrib, Format.attrib, LineCol.attrib, Anchor.attrib,
                   Tag.attrib, merge_attrib]:
             if hasattr(self, a):
@@ -361,6 +367,7 @@ class CommentedBase(object):
                     setattr(t, a, copy.deepcopy(getattr(self, a)))
                 else:
                     setattr(t, a, getattr(self, a))
+        # fmt: on
 
     def _yaml_add_eol_comment(self, comment, key):
         # type: (Any, Any) -> None
@@ -376,7 +383,7 @@ class CommentedBase(object):
 
 
 class CommentedSeq(list, CommentedBase):
-    __slots__ = Comment.attrib,
+    __slots__ = (Comment.attrib,)
 
     def _yaml_add_comment(self, comment, key=NoComment):
         # type: (Any, Optional[Any]) -> None
@@ -404,7 +411,7 @@ class CommentedSeq(list, CommentedBase):
 
     def pop(self, idx=None):
         # type: (Any) -> Any
-        res = list.pop(self, idx)  # type: ignore
+        res = list.pop(self, idx)
         self.ca.items.pop(idx, None)  # might not be there -> default value
         for list_index in sorted(self.ca.items):
             if list_index < idx:
@@ -423,7 +430,7 @@ class CommentedSeq(list, CommentedBase):
             sel_idx = post
         else:
             # self.ca.items is not ordered
-            for row_idx, k1 in enumerate(self):
+            for row_idx, _k1 in enumerate(self):
                 if row_idx >= key:
                     break
                 if row_idx not in self.ca.items:
@@ -455,15 +462,18 @@ class CommentedSeq(list, CommentedBase):
         # type: (Any, Any) -> None
         # try to preserve the scalarstring type if setting an existing key to a new value
         if idx < len(self):
-            if isinstance(value, string_types) and \
-               not isinstance(value, ScalarString) and \
-               isinstance(self[idx], ScalarString):
+            if (
+                isinstance(value, string_types)
+                and not isinstance(value, ScalarString)
+                and isinstance(self[idx], ScalarString)
+            ):
                 value = type(self[idx])(value)
         list.__setitem__(self, idx, value)
 
 
 class CommentedKeySeq(tuple, CommentedBase):
     """This primarily exists to be able to roundtrip keys that are sequences"""
+
     def _yaml_add_comment(self, comment, key=NoComment):
         # type: (Any, Optional[Any]) -> None
         if key is not NoComment:
@@ -490,7 +500,7 @@ class CommentedKeySeq(tuple, CommentedBase):
             sel_idx = post
         else:
             # self.ca.items is not ordered
-            for row_idx, k1 in enumerate(self):
+            for row_idx, _k1 in enumerate(self):
                 if row_idx >= key:
                     break
                 if row_idx not in self.ca.items:
@@ -511,7 +521,7 @@ class CommentedKeySeq(tuple, CommentedBase):
 
 
 class CommentedMapView(Sized):
-    __slots__ = '_mapping',
+    __slots__ = ('_mapping',)
 
     def __init__(self, mapping):
         # type: (Any) -> None
@@ -520,23 +530,8 @@ class CommentedMapView(Sized):
     def __len__(self):
         # type: () -> int
         count = len(self._mapping)
-        done = []  # type: List[Any] # list of processed merge items, kept for masking
-        for merged in getattr(self._mapping, merge_attrib, []):
-            for x in merged[1]:
-                if self._mapping._unmerged_contains(x):
-                    continue
-                for y in done:
-                    if x in y:
-                        break
-                else:
-                    count += 1
-            done.append(merged[1])
         return count
 
-    def __repr__(self):
-        # type: () -> str
-        return '{0.__class__.__name__}({0._mapping!r})'.format(self)
-
 
 class CommentedMapKeysView(CommentedMapView, Set):
     __slots__ = ()
@@ -552,6 +547,7 @@ class CommentedMapKeysView(CommentedMapView, Set):
 
     def __iter__(self):
         # type: () -> Any  # yield from self._mapping  # not in py27, pypy
+        # for x in self._mapping._keys():
         for x in self._mapping:
             yield x
 
@@ -592,12 +588,16 @@ class CommentedMapValuesView(CommentedMapView):
 
     def __iter__(self):
         # type: () -> Any
-        for key in self._mapping:
+        for key in self._mapping._keys():
             yield self._mapping[key]
 
 
-class CommentedMap(ordereddict, CommentedBase):
-    __slots__ = Comment.attrib,
+class CommentedMap(CommentedBase, MutableMapping):
+    __slots__ = (Comment.attrib, '_od')
+
+    def __init__(self, *args, **kw):
+        # type: (Any, Any) -> None
+        self._od = ordereddict(*args, **kw)
 
     def _yaml_add_comment(self, comment, key=NoComment, value=NoComment):
         # type: (Any, Optional[Any], Optional[Any]) -> None
@@ -637,7 +637,7 @@ class CommentedMap(ordereddict, CommentedBase):
             sel_idx = post
         else:
             # self.ca.items is not ordered
-            for row_idx, k1 in enumerate(self):
+            for k1 in self:
                 if k1 >= key:
                     break
                 if k1 not in self.ca.items:
@@ -659,7 +659,7 @@ class CommentedMap(ordereddict, CommentedBase):
     def update(self, vals):
         # type: (Any) -> None
         try:
-            ordereddict.update(self, vals)
+            self._od.update(vals)
         except TypeError:
             # probably a dict that is used
             for x in vals:
@@ -670,7 +670,7 @@ class CommentedMap(ordereddict, CommentedBase):
         """insert key value into given position
         attach comment if provided
         """
-        ordereddict.insert(self, pos, key, value)
+        self._od.insert(pos, key, value)
         if comment is not None:
             self.yaml_add_eol_comment(comment, key=key)
 
@@ -703,7 +703,7 @@ class CommentedMap(ordereddict, CommentedBase):
     def __getitem__(self, key):
         # type: (Any) -> Any
         try:
-            return ordereddict.__getitem__(self, key)
+            return self._od.__getitem__(key)
         except KeyError:
             for merged in getattr(self, merge_attrib, []):
                 if key in merged[1]:
@@ -714,21 +714,23 @@ class CommentedMap(ordereddict, CommentedBase):
         # type: (Any, Any) -> None
         # try to preserve the scalarstring type if setting an existing key to a new value
         if key in self:
-            if isinstance(value, string_types) and \
-               not isinstance(value, ScalarString) and \
-               isinstance(self[key], ScalarString):
+            if (
+                isinstance(value, string_types)
+                and not isinstance(value, ScalarString)
+                and isinstance(self[key], ScalarString)
+            ):
                 value = type(self[key])(value)
-        ordereddict.__setitem__(self, key, value)
+        self._od.__setitem__(key, value)
 
     def _unmerged_contains(self, key):
         # type: (Any) -> Any
-        if ordereddict.__contains__(self, key):
+        if self._od.__contains__(key):
             return True
         return None
 
     def __contains__(self, key):
         # type: (Any) -> bool
-        if ordereddict.__contains__(self, key):
+        if self._od.__contains__(key):
             return True
         # this will only work once the mapping/dict is built to completion
         for merged in getattr(self, merge_attrib, []):
@@ -746,13 +748,13 @@ class CommentedMap(ordereddict, CommentedBase):
     def __repr__(self):
         # type: () -> Any
         if not hasattr(self, merge_attrib):
-            return ordereddict.__repr__(self)
+            return self._od.__repr__()
         return 'ordereddict(' + repr(list(self._items())) + ')'
 
     def non_merged_items(self):
         # type: () -> Any
-        for x in ordereddict.__iter__(self):
-            yield x, ordereddict.__getitem__(self, x)
+        for x in self._od.__iter__():
+            yield x, self._od.__getitem__(x)
 
     def __delitem__(self, key):
         # type: (Any) -> None
@@ -764,19 +766,19 @@ class CommentedMap(ordereddict, CommentedBase):
             except KeyError:
                 pass
         try:
-            ordereddict.__delitem__(self, key)
+            self._od.__delitem__(key)
         except KeyError:
             if not found:
                 raise
 
     def __iter__(self):
         # type: () -> Any
-        for x in ordereddict.__iter__(self):
+        for x in self._od.__iter__():
             yield x
         done = []  # type: List[Any]  # list of processed merge items, kept for masking
         for merged in getattr(self, merge_attrib, []):
             for x in merged[1]:
-                if ordereddict.__contains__(self, x):
+                if self._od.__contains__(x):
                     continue
                 for y in done:
                     if x in y:
@@ -787,12 +789,12 @@ class CommentedMap(ordereddict, CommentedBase):
 
     def _keys(self):
         # type: () -> Any
-        for x in ordereddict.__iter__(self):
+        for x in self._od.__iter__():
             yield x
         done = []  # type: List[Any]  # list of processed merge items, kept for masking
         for merged in getattr(self, merge_attrib, []):
-            for x in merged[1]:
-                if ordereddict.__contains__(self, x):
+            for x in merged[1].keys():
+                if self._od.__contains__(x):
                     continue
                 for y in done:
                     if x in y:
@@ -801,7 +803,28 @@ class CommentedMap(ordereddict, CommentedBase):
                     yield x
             done.append(merged[1])
 
+    def __len__(self):
+        # type: () -> int
+        count = self._od.__len__()  # type: int
+        done = []  # type: List[Any] # list of processed merge items, kept for masking
+        for merged in getattr(self, merge_attrib, []):
+            for x in merged[1]:
+                if self._od.__contains__(x):
+                    continue
+                for y in done:
+                    if x in y:
+                        break
+                else:
+                    count += 1
+            done.append(merged[1])
+        return count
+
+    def __eq__(self, other):
+        # type: (Any) -> bool
+        return bool(dict(self) == other)
+
     if PY2:
+
         def keys(self):
             # type: () -> Any
             return list(self._keys())
@@ -813,28 +836,31 @@ class CommentedMap(ordereddict, CommentedBase):
         def viewkeys(self):
             # type: () -> Any
             return CommentedMapKeysView(self)
+
     else:
+
         def keys(self):
             # type: () -> Any
             return CommentedMapKeysView(self)
 
-    def _values(self):
-        # type: () -> Any
-        for x in ordereddict.__iter__(self):
-            yield ordereddict.__getitem__(self, x)
-        done = []  # type: List[Any]  # list of processed merge items, kept for masking
-        for merged in getattr(self, merge_attrib, []):
-            for x in merged[1]:
-                if ordereddict.__contains__(self, x):
-                    continue
-                for y in done:
-                    if x in y:
-                        break
-                else:
-                    yield ordereddict.__getitem__(merged[1], x)
-            done.append(merged[1])
-
     if PY2:
+
+        def _values(self):
+            # type: () -> Any
+            for x in self._od.__iter__():
+                yield self._od.__getitem__(x)
+            done = []  # type: List[Any]  # list of processed merge items, kept for masking
+            for merged in getattr(self, merge_attrib, []):
+                for x in merged[1]:
+                    if self._od.__contains__(x):
+                        continue
+                    for y in done:
+                        if x in y:
+                            break
+                    else:
+                        yield merged[1]._od.__getitem__(x)
+                done.append(merged[1])
+
         def values(self):
             # type: () -> Any
             return list(self._values())
@@ -846,28 +872,31 @@ class CommentedMap(ordereddict, CommentedBase):
         def viewvalues(self):
             # type: () -> Any
             return CommentedMapValuesView(self)
+
     else:
+
         def values(self):
             # type: () -> Any
             return CommentedMapValuesView(self)
 
     def _items(self):
         # type: () -> Any
-        for x in ordereddict.__iter__(self):
-            yield x, ordereddict.__getitem__(self, x)
+        for x in self._od.__iter__():
+            yield x, self._od.__getitem__(x)
         done = []  # type: List[Any]  # list of processed merge items, kept for masking
         for merged in getattr(self, merge_attrib, []):
             for x, v in merged[1].items():
-                if ordereddict.__contains__(self, x):
+                if self._od.__contains__(x):
                     continue
                 for y in done:
                     if x in y:
                         break
                 else:
-                    yield x, v  # ordereddict.__getitem__(merged[1], x)
+                    yield x, v  # self._od.__getitem__(merged[1], x)
             done.append(merged[1])
 
     if PY2:
+
         def items(self):
             # type: () -> Any
             return list(self._items())
@@ -879,7 +908,9 @@ class CommentedMap(ordereddict, CommentedBase):
         def viewitems(self):
             # type: () -> Any
             return CommentedMapItemsView(self)
+
     else:
+
         def items(self):
             # type: () -> Any
             return CommentedMapItemsView(self)
@@ -891,6 +922,13 @@ class CommentedMap(ordereddict, CommentedBase):
             setattr(self, merge_attrib, [])
         return getattr(self, merge_attrib)
 
+    def copy(self):
+        # type: () -> Any
+        x = {}  # update doesn't work
+        for k, v in self._items():
+            x[k] = v
+        return x
+
     def add_yaml_merge(self, value):
         # type: (Any) -> None
         self.merge.extend(value)
@@ -906,11 +944,11 @@ class CommentedMap(ordereddict, CommentedBase):
 
 
 class CommentedOrderedMap(CommentedMap):
-    __slots__ = Comment.attrib,
+    __slots__ = (Comment.attrib,)
 
 
-class CommentedSet(MutableSet, CommentedMap):
-    __slots__ = Comment.attrib, 'odict',
+class CommentedSet(MutableSet, CommentedMap):  # NOQA
+    __slots__ = Comment.attrib, 'odict'
 
     def __init__(self, values=None):
         # type: (Any) -> None
@@ -949,11 +987,17 @@ class CommentedSet(MutableSet, CommentedMap):
 
 class TaggedScalar(CommentedBase):
     # the value and style attributes are set during roundtrip construction
+    def __init__(self):
+        # type: () -> None
+        self.value = None
+        self.style = None
+
     def __str__(self):
+        # type: () -> Any
         return self.value
 
 
-def dump_comments(d, name='', sep='.', out=sys.stdout):
+def dump_comments(d, name="", sep='.', out=sys.stdout):
     # type: (Any, str, str, Any) -> None
     """
     recursively dump comments, all but the toplevel preceded by the path
@@ -970,5 +1014,6 @@ def dump_comments(d, name='', sep='.', out=sys.stdout):
             print(name)
         print(d.ca, file=out)  # type: ignore
         for idx, k in enumerate(d):
-            dump_comments(k, name=(name + sep + str(idx)) if name else str(idx),
-                          sep=sep, out=out)
+            dump_comments(
+                k, name=(name + sep + str(idx)) if name else str(idx), sep=sep, out=out
+            )
